-----------------------------------------------------------------------------

local TagCompiler = AceLibrary("TagCompiler-1.0")

-----------------------------------------------------------------------------

Aloft.TagData = 
{ 
	TagError = function(x) Aloft:Print(x) end, 
--	TagDebug = function(x) Aloft:Debug(x) end 
}

-----------------------------------------------------------------------------

for k,v in pairs(TagCompiler.CommonTagData) do
	Aloft.TagData[k] = v
end

-----------------------------------------------------------------------------

Aloft.TagData.healthbardeficit = 
{
	events = "Aloft:OnHealthBarValueChanged",
	compileString = function(dataTable, prior) return string.format("%s%s(data.healthBarMaxValue - data.healthBarValue)", prior or "", prior and " and " or "") end,
	guaranteeResult = true,
	numeric = true
}

Aloft.TagData.healthbarmaxvalue = 
{
	events = "Aloft:OnHealthBarValueChanged",
	tableData = "healthBarMaxValue",
	guaranteeResult = true,
	numeric = true
}

Aloft.TagData.healthbarvalue = 
{
	events = "Aloft:OnHealthBarValueChanged",
	tableData = "healthBarValue",
	guaranteeResult = true,
	numeric = true
}

Aloft.TagData.healthpercent = 
{
	events = "Aloft:OnHealthBarValueChanged",
	compileString = function(dataTable, prior) return string.format("%s%sfloor(100*data.healthBarValue / data.healthBarMaxValue)", prior or "", prior and " and " or "") end,
	guaranteeResult = true,
	numeric = true
}

Aloft.TagData.isgroup =
{
	compileString = function(dataTable, prior) return string.format("data.healthBarMaxValue ~= 100%s%s", prior and " and " or "", prior or "") end,
	noGuaranteeChange = true
}

Aloft.TagData.isnotgroup = 
{
	compileString = function(dataTable, prior) return string.format("data.healthBarMaxValue == 100%s%s", prior and " and " or "", prior or "") end,
	noGuaranteeChange = true
}

Aloft.TagData.isboss = { tableData = "isBoss" }

Aloft.TagData.isfriendly =
{
	compileString = function(dataTable, prior)
						return string.format("(data.type == \"friendlyPlayer\" or data.type == \"friendlyNPC\")%s%s", prior and " and " or "", prior or "")
					end
}

Aloft.TagData.isfriendlynpc =
{
	compileString = function(dataTable, prior) return string.format("data.type == \"friendlyNPC\"%s%s", prior and " and " or "", prior or "") end
}

Aloft.TagData.isfriendlyplayer =
{
	compileString = function(dataTable, prior) return string.format("data.type == \"friendlyPlayer\"%s%s", prior and " and " or "", prior or "") end
}

Aloft.TagData.ishostile =
{
	compileString = function(dataTable, prior) return string.format("data.type == \"hostile\"%s%s", prior and " and " or "", prior or "") end
}

Aloft.TagData.isneutral =
{
	compileString = function(dataTable, prior) return string.format("data.type == \"neutral\"%s%s", prior and " and " or "", prior or "") end
}

Aloft.TagData.ismaxhealth =
{
	events = "Aloft:OnHealthBarValueChanged",
	compileString = function(dataTable, prior) return string.format("data.healthBarValue == data.healthBarMaxValue%s%s", prior and " and " or "", prior or "") end
}

Aloft.TagData.level = { tableData = "level", numeric = true }

Aloft.TagData.name = { tableData = "name", guaranteeResult = true }

Aloft.TagData.difficultycolor =
{
	method = function(aloftData, value) return value and ("|cff%02x%02x%02x%s|r"):format(aloftData.levelTextR*255, aloftData.levelTextG*255, aloftData.levelTextB*255, value) end,
	noGuaranteeChange = true
}

-----------------------------------------------------------------------------

-- Initialise the compile cache with an empty tag, something which would otherwise need some extra special-cased code
local compileCache =
{
	[""] = { data = { }, events = { }, tag = "", method = function() return "" end },
}

function Aloft:CreateTag(tag)
	local parseTag = tag or ""

	if compileCache[parseTag] then
		return compileCache[parseTag]
	end

	local result = TagCompiler:Compile(tag, self.TagData, "string")
	compileCache[tag] = result
	return result
end

-----------------------------------------------------------------------------
